use POSIX;
use File::Path;
use Mail::Sender;

#sub email_notification($$);
#sub email_voicemail_file($$$);

sub collect_vmail($) {
	my $ext=shift;
	my ($email, $vmaildir, $email_vm, $delete_on_email, $email_notification);
	my $maxmsg;

	$ctport->logger("collect_voicemail: Entering Collect Voicemail State");

	# load up all the params we need

	$email = $ctport->{CONFIG}{$ext}{"email"};
	$email = "" unless defined($email);
	$vmaildir = $ctport->{CONFIG}{$ext}{"label"};
	$email_vm = $ctport->{CONFIG}{$ext}{"email_voicemail"};
	$email_vm = "no" unless defined($email_vm);
	$email_pin = $ctport->{CONFIG}{$ext}{"pin"};
	$delete_on_email = $ctport->{CONFIG}{$ext}{"delete_on_email"};
	$delete_on_email = "no" unless defined($delete_on_email);
	$email_notification = $ctport->{CONFIG}{$ext}{"email_notification"};
	$email_notification = "no" unless defined($email_notification);
	$maxmsg = $ctport->{CONFIG}{"voicemail"}{"maxmsg"};
	$maxmsg = 100 unless defined($maxmsg);
	$ctport->logger("collect_voicemail: $ext $vmaildir $email $email_vm "
			. "$delete_on_email $email_notification");

	my $greetfile="$vmailgreetpath"."/greet-".$vmaildir.".wav";

	# default prompt if no greeting record yet
	if (!(-e $greetfile)) {
		$ctport->logger("collect_voicemail: no greeting file $greetfile, using default");
		$greetfile = "$promptpath/voicemsg3.wav";
	}
	$ctport->logger("collect_voicemail: playing greeting $greetfile");

	my $messagedir="$vmailmailpath/$vmaildir";

	# test vmail file path OK 
	if (!(-d $messagedir)) {
		# sanity check, in case conf file wrong
		if (!defined($vmaildir)) {
			$ctport->logger("collect_voicemail: ERROR label not defined");
			return "";
		}

		$ctport->logger("collect_voicemail: vmail dir doesnt exist, creating");
		eval { mkpath($messagedir) };
		if ($@) {
			$ctport->logger("collect_voicemail: couldn't create $dir: $@");
		}
	}

	# check if mailbox full
	
	my @msglist = build_message_list($messagedir);
	my $nummsg = $#msglist+1;
	if ($nummsg >= $maxmsg) {
		$ctport->logger("collect_voicemail: Too many messages, limit $maxmsg");
		return "";
	}
					 
	my $tstamp = strftime("%Y%m%d%H%M%S",localtime(time));
	my $messagefile=$messagedir."/".$tstamp.".wav";

	$ctport->clear();
	my $ret = $ctport->play("$greetfile beep");

	# they might hangup during prompt, in this case dont leave message
	if (defined($ctport->event)) {
		# station hang up or flash
		if ($ctport->event eq "hangup") {
			return $ctport->event;
		}
		elsif ($ctport->event eq "flash") {
			return $ctport->event;
		}
		# trunk port hang up
		elsif ($ctport->event eq "loop drop") {
			return $ctport->event;
		}
		else {
			$ctport->logger("collect_voicemail: Got [".$ctport->event."] "
					. "while playing greeting");
		}
	}
	if ($ret eq "DTMF"){
		$ctport->logger("collect_voicemail: Got DMTF ");
		$digits=$ctport->collect(4,15,5);
		if ($digits eq $email_pin){
			$ctport->logger("collect_voicemail: Got pin! ");
			$ret = voicemail_listen_msgs($vmaildir);
			return $ret;
		}
	}

	
	$ctport->logger("collect_voicemail: Recording to $messagefile");
	$ctport->record($messagefile,30,"#");
	$ctport->logger("collect_voicemail: Done");
	
	# handle email options

	if ($email_vm eq "yes") {
		$ctport->logger("collect_voicemail: about to email vm");
		email_voicemail_file($email, $ext, $messagefile);
		if ($delete_on_email eq "yes") {
			$ctport->logger("collect_voicemail: about to delete vm file");
			unlink $messagefile;
		}
	}
	if ($email_notification eq "yes") {
		$ctport->logger("collect_voicemail: about to email notification");
		email_notification($email, $ext);
	}

	# just return for now, handle AA and other possibilities later

	if (defined($ctport->event)) {
		# station hang up or flash
		if ($ctport->event eq "hangup") {
			return $ctport->event;
		}
		if ($ctport->event eq "flash") {
			return $ctport->event;
		}

		# trunk port hang up
		if ($ctport->event eq "loop drop") {
			return $ctport->event;
		}
	}

	return "";
}

sub sig_hup{
	local $SIG{HUP}='IGNORE';
	load_config();
	$ctport->logger("handle_signal: Reloaded configuration file");
	return;
}

sub procargs(@){
        $ME = 0;
	while($bit = shift @ARGV){
		if ($bit =~ /-d/){
			$daemon=1;
		}
		elsif($bit =~ /-p/){
			$ME = shift @ARGV;
			$port = 1200 + $ME;
		}
		elsif($bit =~ /-c/){
			$configfile = shift @ARGV;
		}
	}
}

sub roll_log(){
	$ctport->logger("Rolling log file");
	$ctport->closelogger();
	$ctport->openlogger($ctport->{CONFIG}{LOGFILE});
	$ctport->logger("Log file rolled!");
}

sub load_config(){
	$ctport->getconf($configfile);
	foreach my $bit (keys %{$ctport->{CONFIG}{"dialmap"}}){
		my $port=$ctport->{CONFIG}{"dialmap"}{$bit};
		$ctport->{CONFIG}{"dialmap-rev"}{$port}=$bit;
	}
}

sub auto_attendant_answer() {
	my ($event_source,$event);
	my $ringstoanswer = $ctport->{CONFIG}{"autoattendant"}{"ringstoanswer"};
	my $rings = 1;

	$ctport->logger("auto_attendant_answer: Entering State");
	$ctport->start_timer_async(10);

	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("handle_idle: event $event");

		if ($event eq "ring") {
			$rings++;
			if ($rings == $ringstoanswer) {
				return "handle autoattendant";
			}
			$ctport->start_timer_async(10);
		}
		elsif ($event eq "timer") {
			return "handle idle";
		}
	}
}

sub auto_attendant(){
	my ($digit, $who, $event,$ext, $loops);
	my $annoucement = $ctport->{CONFIG}{"autoattendant"}{"prompt"};
	my $repeats = $ctport->{CONFIG}{"autoattendant"}{"repeatprompt"};

	$ctport->logger("auto_attendant: Entering Auto Attendant State");

	# fix for spurious loop drop event on off hook transient
	# wait one second, discarding events

	$ctport->start_timer_async(1);
	($who,$event) = $ctport->wait_for_event(); 
	while ($event ne "timer") {
		($who,$event) = $ctport->wait_for_event(); 
	}
	
	if (defined($ctport->event)) {
		$ctport->clear();
	}
	
	# OK, main AA loop
	
	$loops = 0;
	while(1) {
		$ctport->play($annoucement);
		$ctport->ctsleep(5);

		if (defined($ctport->event)) {
			$ctport->logger("auto_attendant: event " . $ctport->event);
			$digit = $ctport->event;
		}
		else { 
			$digit = "";
	        }

		if ($digit eq "*") {
			$digits=$ctport->collect(1,15,5);
			$ctport->logger("auto_attendant: * Pressed");
			$digits=$ctport->collect(3,15,5);
			$port = $ctport->{CONFIG}{$digits}{"port"};
			if (defined($port)) {
				# port exits, set up group to return to AA if
				# ring out and no VM
				$group = "$digits,aa";
				return "handle group call";
			}
			else {
				$ctport->logger("auto_attendant: Couldnt find [$extension]");
			}
		}
		elsif ($digit eq "loop drop"){
			return "handle wait for hangup";
		}
		elsif ($digit =~ /[0-9]/) {
			# valid AA digit 1,2,3... pressed
			
			my $group_name = $ctport->{CONFIG}{"autoattendant"}{$digit};
			$group = $ctport->{CONFIG}{"groups"}{$group_name};
			$group.=",aa";
			if (defined($group)) {
				return "handle group call";
			}
			else {
				$ctport->logger("auto_attendant: AA Option for $digit undefined");
			}
		}

		# just in case hangup detect fails
		$loops++;
		if ($loops == $repeats) {
			return "handle idle";
		}
	}
}

sub save_profile($$$$$){
	my ($profileid,$dialplan,$pin,$email,$vmail)=@_;
	$ctport->{CONFIG}{"profiles"}{$profileid}="$dialplan,$pin,$email,$vmail";
	$ctport->save_config;
}

sub email_voicemail_file($$$) {
    my($to, $ext, $recfile) = @_;
    my($smtp, $sender);

    $smtp = $ctport->{CONFIG}{"voicemail"}{"smtp"};
    if (!defined($smtp)) {
	    $ctport->logger("email_voicemail_file: ERROR, [voicemail] smtp not defined");
	    return;
    }
    $sender = new Mail::Sender{smtp => $smtp, from => 'OpenPBX', fakefrom => 'OpenPBX'};

    # ret codes defined in perldoc Mail::Sender
    $ret = $sender->MailFile({to => $to, 
                              subject => "Voicemail for $ext",
			      msg => "See attached wave file",
			      file => $recfile});

    if ($ret != $sender) {
	    $ctport->logger("email_voicemail_file: ERROR emailing message, ret = $ret");
    }
    else {
	    $ctport->logger("email_voicemail_file: message emailed OK");
    }

}

sub email_notification($$) {
    my($to, $ext) = @_;
    my($smtp, $sender);

    $smtp = $ctport->{CONFIG}{"voicemail"}{"smtp"};
    if (!defined($smtp)) {
	    $ctport->logger("email_voicemail_file: ERROR, [voicemail] smtp not defined");
	    return;
    }
    $sender = new Mail::Sender{smtp => $smtp, from => 'OpenPBX', fakefrom => 'OpenPBX'};

    # ret codes defined in perldoc Mail::Sender
    $ret = $sender->MailMsg({to => $to, 
                              subject => "New Voicemail for $ext",
			      msg => "Dial *11 to listen to message"}
			      );

    if ($ret != $sender) {
	    $ctport->logger("email_voicemail_file: ERROR emailing message, ret = $ret");
    }
    else {
	    $ctport->logger("email_voicemail_file: message emailed OK");
    }

}

sub build_message_list($) {
	my $vdir = shift;
	my @msglist;

	$ctport->logger("build_message_list: opening $vdir");
	if (opendir MSGDIR, $vdir){
		@msglist=grep /wav/, readdir MSGDIR;
		$ctport->logger("voicemail_menu: found msgs @msglist");
		closedir MSGDIR;
	}
	else {
		$ctport->logger("voicemail_menu: ERROR cant open mail directory");
		return;
	}

	return @msglist;
}


sub voicemail_listen_msgs($) {
	my $vdir=shift;
	my $say;
	my $nummsg;
	my $msg_index;
	my @msglist;
	my ($full, $last);
 
	$ctport->logger("voicemail_listen_msgs: Entering Voicemail Listen to Messages State");

	$ctport->clear;
	while(1) {
		# build new message list for this iteration as we may have deleted some
		@msglist = build_message_list($vmailmailpath . "/" . $vdir);
		
		# initial prompt, number of messages
		$nummsg = $#msglist+1;
		$say = say_number_of_messages($nummsg);
		#$ctport->logger("voicemail_listen_msgs: say = $say");
		$ctport->play($say);
		$ctport->ctsleep(1);          # Wait 1s unless event occurs

		# trap hangup

		if (defined($ctport->event)) {
			if ($ctport->event eq "hangup") {
				return;
			}
			if ($ctport->event eq "loop drop") {
				return $ctport->event;
			}

			# if no msg ignore any DTMFs
			if ($nummsg == 0) {
				$ctport->clear; 
			}
		}

		$msg_index = -1;
		if ($nummsg > 0) {
			$msg_index++;
			$message = $msglist[$msg_index];
			$full = $vmailmailpath . "/" . $vdir . "/" . $message;
			$ctport->play($full);
		}

	        while($msg_index < $nummsg) {

			# menu options
			$ctport->play("$promptpath/voicemlm1.wav");
			$ctport->ctsleep(5);          # Wait 5s unless event occurs
			      
			if (defined($ctport->event)) {
				if ($ctport->event eq "1") {
					$ctport->logger("voicemail_listen_msgs: Menu 1 - Play Next");
					$ctport->clear;               

					$msg_index++;
					if ($msg_index < $nummsg) {
						$message = $msglist[$msg_index];
						$full = $vmailmailpath . "/" . $vdir . "/" . $message;
		#				$ctport->logger("msg_index $msg_index new full $full"); 
						$ctport->play($full);
					}
				}	
				elsif ($ctport->event eq "2") {
					$ctport->logger("voicemail_listen_msgs: Menu 2 - " .
							"Delete & Play Next");
					$ctport->clear;
					if ($msg_index != -1) {
						$ctport->logger("voicemail_listen_msgs: deleting $full");
						unlink $full;
						$ctport->play("$promptpath/voicemlm2.wav");
						$msg_index++;
						if ($msg_index < $nummsg) {
							$message = $msglist[$msg_index];
							$full = $vmailmailpath . "/" . $vdir . "/" . $message;
						}
					}
			        }
				elsif ($ctport->event eq "3") {
					$ctport->logger("voicemail_listen_msgs: Menu 3 - Play Again");
					$ctport->clear;               
					if ($msg_index != -1) {
						$ctport->logger("voicemail_listen_msgs: last $full");
						if (-e $full) {
							$ctport->play($full);
						}
					}
				}
				elsif ($ctport->event eq "hangup") {
					return; 
			        }
				elsif ($ctport->event eq "loop drop") {
					if ($ctport->{CONFIG}{"ports"}{$aparty} =~/trunk/){
						return; 
					}
					else {
						$ctport->clear;
					}
			        }
				else {
					$ctport->clear;
				}
			}
		}

		# force us to listen to message count on next loop
		$ctport->clear;               

	}
}

sub voicemail_greeting_menu($) {
	my $greetfile = shift;
	my ($say, $cur_msg);
    
	$ctport->clear;               

	while(1) {
		$ctport->play("$promptpath/voicemg1.wav");
		$ctport->ctsleep(5);          # Wait 5s unless event occurs
		
		if (defined($ctport->event)) {
			if ($ctport->event eq "1") {       
				$ctport->logger("voicemail_greeting_menu: Menu 1 - Record Greeting");

				$ctport->clear;               
				$ctport->play("$promptpath/voicemsg2.wav beep.au");
				$ctport->record($greetfile,30,"#");
			}	
			elsif ($ctport->event eq "2") {
				$ctport->logger("voicemail_greeting_menu: Menu 2 - Play Greeting");
				$ctport->clear;               
				$ctport->play($greetfile);
			}
			elsif ($ctport->event eq "hangup") { 
				return; 
			}
			else {
				$ctport->clear;
			}
		}
	}
}

sub say_number_of_messages($) {
	my $num_msg = shift;
	my $say;

	if ($num_msg == 0) {
		$say = "$promptpath/voicem2a.wav";
	}
	if ($num_msg == 1) {
		$say = "$promptpath/voicem2b.wav";
	}
        elsif ($num_msg > 1) {
	        $say = "$promptpath/voicem2.wav $num_msg $promptpath/voicem3.wav";	
	}
	return $say;
}

1;
