#!/usr/bin/perl -w
# station.pl
# VoiceTronix 2003
#
# ctserver script for PBX station port


use Telephony::CTPort;
use Data::Dumper;
use Cwd qw(cwd);
use common;
use File::Path;


# function prototypes ------------------------------------

sub generate_hook_flash();
sub handle_incoming_call();
sub handle_dial_menu();
sub star_menu();
sub main_loop();
sub port_busy($);
sub collect_internal_number($);
sub make_internal_call();
sub ring_internal_call();
sub make_external_call();
sub check_bparty($);
sub handle_call();
sub handle_wait_for_hangup();
sub unhold($);
sub handle_onhold();
sub handle_flash();
sub voicemail_menu();
sub say_number_of_messages($);
sub get_lcr($);
sub get_trunk($);
sub voicemail_greet_man($);
sub voicemail_listen_msgs($);
sub build_message_list($);
sub station_manager();
sub login_profile();
sub logout_profile($);
sub grab_ringing_extension();
sub is_trunk_ringing($);

# constants --------------------------------------------------------

our $ext_line_key;            # DTMF key to get external line, e.g. 9 or 0
our $configfile="pbx.conf";   # default config file
our $station_ring_pause = 3;
our $port=1200;
our $daemon=0;

# globals ----------------------------------------------------------

our $ctport;
our $aparty = -1;	# its who I am !!
our $bparty     = -1;   # handle for other end of current phone call
our $hold_party = -1;   # handle for party we have on hold, -1 otherwise
our $dtmf;              # dtmf digits dialled, e.g. extension to call
our $internal_number;   # dtmf digits dialled, e.g. extension to call
our $promptpath;        # path to all audio prompts
our $vmailgreetpath;    # path to voice mail greetings
our $vmailmailpath;     # path to actual mail messages
our $homedir=cwd();		# path to our home dir
		
procargs(@ARGV);
		
# start code here -------------------------------------------------------

$aparty=$port - 1200;
$ctport = new Telephony::CTPort($port);
$ctport->getconf($configfile);
if ($daemon) {
	my $logfile=$homedir."/log/$port-station";
	$ctport->openlogger($logfile);
	$ctport->daemonize($daemon);
}
$ctport->logger("STARTUP: Connecting to CTServer on port $port.");

my $moh_path = $ctport->{CONFIG}{"moh"}{"path"};
if (defined($moh_path)) {
	$ctport->set_paths($moh_path);
}
my @MOH;
foreach my $bit (keys %{$ctport->{CONFIG}{"moh"}}){
	push @MOH, $ctport->{CONFIG}{"moh"}{$bit};
}
foreach my $bit (keys %{$ctport->{CONFIG}{"dialmap"}}){
	my $port=$ctport->{CONFIG}{"dialmap"}{$bit};
	$ctport->{CONFIG}{"dialmap-rev"}{$port}=$bit;
}
$vmailgreetpath = $homedir."/".$ctport->{CONFIG}{"voicemail"}{"greetpath"};
$vmailmailpath = $homedir."/".$ctport->{CONFIG}{"voicemail"}{"mailpath"};
$promptpath = $homedir."/".$ctport->{CONFIG}{"general"}{"path"};
$ext_line_key = $ctport->{CONFIG}{"general"}{"externallinekey"};
$ext_line_key = "0" unless defined($ext_line_key);

$SIG{HUP}=\&sig_hup;

$ctport->set_event_handler("rolllog",\&roll_log);
$ctport->set_event_handler("load_config",\&load_config);

main_loop();

exit;

sub main_loop(){
	my $event_handler = "handle idle";

	while(1) {
		$ctport->setscript($event_handler);
		if ($event_handler eq  "handle idle") {
			$event_handler = handle_idle();
		}
		elsif ($event_handler eq  "handle incoming call") {
			$event_handler = handle_incoming_call();
		}
		elsif ($event_handler eq  "handle dial menu") {
			$event_handler = handle_dial_menu();
		}
		elsif ($event_handler eq  "make internal call") {
			$event_handler = make_internal_call();
		}
		elsif ($event_handler eq  "ring internal call") {
			$event_handler = ring_internal_call();
		}
		elsif ($event_handler eq  "handle call") {
			$event_handler = handle_call();
		}
		elsif ($event_handler eq  "make external call") {
			$event_handler = make_external_call();
		}
		elsif ($event_handler eq  "handle onhold") {
			$event_handler = handle_onhold();
		}
		elsif ($event_handler eq  "handle unhold") {
			$event_handler = handle_unhold();
		}
		elsif ($event_handler eq  "handle wait for hangup") {
			$event_handler = handle_wait_for_hangup();
		}
		elsif ($event_handler eq  "handle station manager") {
			$event_handler = station_manager();
		}
		elsif ($event_handler eq  "handle star menu") {
			$event_handler = star_menu();
		}
		elsif ($event_handler eq  "handle voicemail menu") {
			$event_handler = voicemail_menu();
		}
		else {
			$ctport->logger("main: UNKNOWN HANDLER: $event_handler");
			$event_handler = "handle wait for hangup";
		}

	}
}

# used for reading and configuring voicemail

sub voicemail_menu() {
	my $ext=shift;
	my $dtmf;
	my $greetfile;
	my $stationid;
	my $ret="";
	my @msglist;
	my $result="";
	my ($extension);
	my ($port,$max_rings,$voicemail,$pin,$email,$vdir);
	my ($say, $cur_msg);
	my ($testpath);

	$ctport->logger("voicemail_menu: Entering Voicemail Menu State");

	# given $aparty, determine the extension by scanning the profile list

	if (!defined $ext){
		my $extlist = $ctport->{CONFIG}{"extmap"}{"extensions"};
		foreach $ext (split /,/,$extlist) {
			$port=$ctport->{CONFIG}{$ext}{"port"};
			if ($port eq $aparty){
			    $extension = $ext;
			}
		}
	}
	else {
		$extension=$ext;
	}
	if (!defined $extension) {
		# oops - no profile exists, cant do anything with voicemail, 
		$ctport->clear;
		$ctport->play("$promptpath/voicem1a.wav");
		return "handle wait for hangup";
	}

	$profile=$ctport->{CONFIG}{"profiles"}{$extension};
	$vdir = $ctport->{CONFIG}{$extension}{"label"};
	$ctport->logger("voicemail_menu: found profile for $extension");

	# test vmail file path OK 

	$testpath=$vmailmailpath."/".$vdir;
	if (!(-d ($testpath))) {
		$ctport->logger("voicemail_menu: vmail dir [$testpath] doesnt exist, creating");
		eval { mkpath($testpath) };
		if ($@) {
			$ctport->logger("voicemail_menu: couldn't create [$testpath]: $@");
			return "handle wait for hangup";
		}
	}

	# test greeting file path OK 
	if (!(-d ($vmailgreetpath))) {
		$ctport->logger("voicemail_menu: greeting path doesnt exist, creating");
		eval { mkpath($vmailgreetpath) };
		if ($@) {
			$ctport->logger("voicemail_menu: couldn't create $dir: $@");
			return "handle wait for hangup";
		}
	}

	# determine our greeting file and build a list of current messages

	$greetfile=$vmailgreetpath."/greet-".$vdir.".wav";
	$ctport->logger("voicemail_menu: greetfile: $greetfile");

	while(1) {
		$ctport->clear;               # clears any events and digit buffer
		$ctport->play("$promptpath/voicem1.wav");
		$ctport->ctsleep(5);          # Wait 5s unless event occurs
		
		if (defined($ctport->event)) {
			if ($ctport->event eq "1") {       
				$ctport->logger("voicemail_menu: Menu 1 - Listen to Messages");
				voicemail_listen_msgs($vdir); 
				return "handle idle";
			}	
			if ($ctport->event eq "2") {
				$ctport->logger("voicemail_menu: Menu 2 - Change Greeting");
				voicemail_greeting_menu($greetfile); 
				return "handle idle";
			}
			if ($ctport->event eq "hangup") {
				return "handle idle";
			}
		}
	}

}

sub handle_idle() {
	my ($event_source,$event);

	$ctport->logger("handle_idle: Entering Idle State");
	$bparty = -1;
	$hold_party = -1;
	$ctport->unset_event_handler("incoming_call");
	
	while(1) {
		($event_source,$event) = $ctport->wait_for_event(); 
		$ctport->logger("handle_idle: event $event");

		if ($event eq "incoming_call") {
			$ctport->logger("handle_idle: Incoming Call");
			$ctport->set_event_handler("incoming_call",\&port_busy);
			$ctport->send_event($event_source, "call_accepted");
			$ctport->logger("handle_idle: sent reply to $event_source");
			$bparty = $event_source;
			return "handle incoming call";
		}
		if ($event eq "pickup") {
			$ctport->logger("handle_idle: Handset Pickup");
			$ctport->set_event_handler("incoming_call",\&port_busy);
			return "handle dial menu";
		}
	}
}

sub port_busy($) {
	my $event_source = shift;
	$ctport->logger("port_busy: call not accepted");
	$ctport->send_event($event_source, "call_not_accepted");    
	return;
}

sub handle_incoming_call() {
	my ($who, $event, $extra);

	$ctport->logger("handle_incoming_call: Entering Incoming Call State");
	$ctport->start_timer_async(10);

	while(1) {
		($who,$event,$extra) = $ctport->wait_for_event(); 
		$ctport->logger("handle_incoming_call: event $event");

		if ($event =~ /ring/i) {
			# other port has asked us to ring station
			# ring station for 1 second, reset timer
			$ctport->start_ring_once_async();
			$ctport->start_timer_async(10);
		}
		elsif ($event =~ /pickup/i) {
			# some one has picked up this station
			$ctport->send_event($bparty, "pickup");
			return "handle call";
		}
		elsif ($event eq "timer") {
			# no rings for 10 seconds, assume call has finished
			return "handle idle";
		}
		elsif ($event eq "bye") {
			# other end hangs up before answer
			return "handle idle";
		}
		elsif ($event eq "bparty") {
			# change bparties, e.g. for blind tfer
			$bparty = $extra;
		}
		else {
			$ctport->logger("handle_incoming_call: unknownevent $event ");
		}
		($who,$event)=(-1,"");
	}
}

sub handle_dial_menu() {
	my ($who,$event, $next_event_handler);

	$ctport->logger("handle_dial_menu: Entering Dial Menu State");

	# start dial tone playing
	$ctport->play_dial_tone_async();

	$ctport->clear();
	$ctport->start_timer_async(90);

	# wait for single digit to determine if ext or int call

	$ctport->logger("handle_dial_menu: Waiting for some input");
	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("handle_dial_menu: event $event");
			
		if ($event eq "dtmf") {
			$ctport->play_terminate();
			$ctport->stop_timer();
			$dtmf = $ctport->collect(1,1);
			$ctport->logger("handle_dial_menu: DTMF [$dtmf]");
			if ($dtmf ne "") { 
				$next_event_handler = process_digit($dtmf);
				return $next_event_handler;
			}
			else {
				$ctport->play_dial_tone_async();
			}
		}
		elsif ($event eq "hangup") {
			$ctport->play_terminate();
			$next_event_handler = handle_hangup();
			return $next_event_handler;
		}
		elsif ($event eq "timer") {
			$ctport->play_terminate();
			return "handle wait for hangup";
		}
		elsif ($event eq "flash") {
			$ctport->play_terminate();
			if (hold_party_exists()) {
				$next_event_handler = handle_flash();
				return $next_event_handler;
			}
		}
		elsif ($event eq "toneend") {
			$ctport->play_dial_tone_async();
		}
		elsif ($event eq "bye") {
			$ctport->play_terminate();
			return "handle wait for hangup";
		}
		else {
			$ctport->logger("handle_dial_menu: unknown-event $event");
		}
	}
}
		
sub process_digit($) {
	my $dtmf = shift;
	my $ret;

	if ($dtmf eq $ext_line_key) {
		return "make external call";
	}
	elsif($dtmf eq "*") {
		# star menu can only be entered if we are not in a call
		if (($bparty == -1) && ($hold_party == -1)) {
			return "handle star menu";
		}
		else {
			return "wait for hangup";
		}
        }
	elsif($dtmf eq "9") {
		# send a trunk call straight to an extensions voicemail
		# do nothing for station call

		if ($ctport->{CONFIG}{ports}{$hold_party} eq "trunk") {
			$ctport->clear();
			my $digits = $ctport->collect(3,5);
			
			$ctport->logger("process_digits: vm: $digits");
			if ($digits eq "EVENT") {
				# handle hangup and flash
				if ($ctport->event eq "hangup") {
					return handle_hangup();
				}
				if ($ctport->event eq "flash") {
					return handle_flash();
				}
				    
			}
			else {
				$ctport->send_message($hold_party, ("voicemail", $digits));
				$hold_party = -1;
				return "handle wait for hangup";
			}
		}
		else {
			return "wait for hangup";
		}
        }
	elsif($dtmf eq "#") {
		$ret = generate_hook_flash();
		if ($ret eq "") {
			# if not valid state for external flash just ignore and
			# assume we are performing normal dialling.  User can just
			# flash or hangup to exit.
			return collect_internal_number($dtmf);
		}
		else {
			return $ret;
		}
	}
	else {
		return collect_internal_number($dtmf);
	}
}

sub collect_internal_number($) {

	my $dtmf=shift;
	my ($who, $event, $num_digits);

	$ctport->logger("collect_internal_number: Entering Collect Internal Number State");

	# collect the rest of the digits.  Need an event loop here to handle exceptions

	$num_digits = 1;
	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("make_internal_call: event $event");
			
		if ($event eq "dtmf") {
			$dtmf .= $ctport->collect(1,1);
			$num_digits++;
			if ($num_digits == 3) {
				$internal_number = $dtmf;
				return "make internal call";
			}
		}
		elsif ($event eq "flash") {
			$ctport->play_terminate();
			if (hold_party_exists()) {
				$next_event_handler = handle_flash();
				return $next_event_handler;
			}
		}
	        elsif ($event eq "hangup") {
		        $next_event_handler = handle_hangup();
		        return $next_event_handler;
	        }

	}
}

# given a number to dial, try to make internal call

sub make_internal_call() {
	my $dtmf = $internal_number;
	my ($who, $event, $tmp, $dp, $ret);
	my ($port,$max_rings,$voicemail,$pin,$email,$vdir);

	# is number dialled valid?

	$port = $ctport->{CONFIG}{$dtmf}{"port"};
	if (!defined($port)) {
		# bad extension, not in dial plan
		$ctport->logger("make_internal_call: bad extension[$dtmf], not in dial plan");
		return "handle wait for hangup";
	}

	# look up port from profile info

	$voicemail = $ctport->{CONFIG}{$dtmf}{"voicemail"};
	$voicemail = "no" unless defined($voicemail);
	$max_rings = $ctport->{CONFIG}{$dtmf}{"ringout"};
	$max_rings = 100 unless defined($max_rings);
	$ctport->logger("make_internal_call: Found info for $dtmf");

	$ctport->logger("make_internal_call: Checking port [$port]");
	if ($port eq "-1"){
	       return "handle wait for hangup";
	}
	elsif ($port ne ""){
                # lets try this port as a potential bparty

		$tmp=$port;
		$ctport->logger("make_internal_call: checking station port [$tmp]");

		# see if bparty is busy
		$ret = check_bparty($tmp);
		if ($ret == 0) {
		        $ctport->logger("make_internal_call: $tmp busy");
                
                        # collect voicemail if we arent dialling ourselves
                        if (($tmp != $aparty) && ($voicemail eq "yes")) {
				$ret = collect_vmail($dtmf);
				if ($ret eq "hangup") {
					return "handle idle";
				}
				if ($ret eq "flash") {
					return handle_flash();
 				}
			}
			return "handle wait for hangup";
		}
		else {
			# success - free port found
			$bparty = $tmp;
		}
        }
	else {
		return "handle wait for hangup";
	}

	return "ring internal call";
}

# given valid bparty, ring other extension

sub ring_internal_call() {
	my ($who, $event, $rings, $tmp, $ret, $extension);
	my ($port,$max_rings,$voicemail,$pin,$email,$vdir, $ext);

	# given $bparty, determine the extension by scanning the profile list

	my $extlist = $ctport->{CONFIG}{"extmap"}{"extensions"};
	foreach $ext (split /,/,$extlist) {
		$port=$ctport->{CONFIG}{$ext}{"port"};
		if ($port eq $bparty){
		        $extension = $ext;
		}
	}

	$voicemail = $ctport->{CONFIG}{$extension}{"voicemail"};
	$voicemail = "no" unless defined($voicemail);
	$max_rings = $ctport->{CONFIG}{$extension}{"ringout"};
	$max_rings = 100 unless defined($max_rings);
	$email = $ctport->{CONFIG}{$extension}{"email"};
	$vdir = $ctport->{CONFIG}{$extension}{"label"};

	# OK, other station has already accepted call, make it ring

	$rings = 1;
	$ctport->send_event($bparty, "ring");
	$ctport->start_timer_async($station_ring_pause);
	$ctport->play_ringback_async();

	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("ring_internal_call: event $event");

		if ($event eq "timer") {
			# timer has fired, ring station again and play ringback tone again

			if ($rings == $max_rings) {
				# we have reached max rings, collect voicemail
				$ctport->send_event($bparty, "bye");
				$bparty = -1;

				if ($voicemail eq "yes") {
					$ret = collect_vmail($extension);
					if ($ret eq "hangup") {
						return handle_hangup();
					}
					if ($ret eq "flash") {
						$ret = handle_flash();
						if ($ret eq "") {
							return "handle wait for hangup";
						}
					}
					else {
						return "handle wait for hangup";
					}
				$voicemail = $ctport->{CONFIG}{$extension}{"voicemail"};
				$voicemail = "no" unless defined($voicemail);
 				}
				
				return "handle wait for hangup";
			}
				
			$ctport->send_event($bparty, "ring");
			$ctport->start_timer_async($station_ring_pause);
			$ctport->play_ringback_async();
			$rings++;
		}
		elsif ($event eq "pickup") {
			# to handle ** - another ext taking this call

			if ($who != $bparty) {
				
				# stop current bparty ringing
				# and change to new bparty
				$ctport->send_event($bparty, "bye");
				$bparty = $who;			
			}

			# handle call
			$ctport->bridge($bparty);
			return "handle call";
		}
		elsif ($event eq "hangup") {
			return handle_hangup(1);
		}
		elsif ($event eq "flash") {
			# hit flash while bparty ringing, tell it call finished
			$ctport->send_event($bparty, "bye");
			$bparty = -1;
			return handle_flash();
		}
		elsif ($event eq "are u ringing") {
			# another ext is attempting ** to grab call
			$ctport->send_event($bparty, "yes");
		}
		else {
			$ctport->logger("ring_internal_call: unknownevent $event");
		}
	}
}

# DR 8/8/03, removed LCR for now to simplify testing for M1 of Release 2

sub make_external_call() {
	my $tmp;

	$ctport->logger("make_external_call: Entering Make External Call State");

	foreach $tmp (keys %{$ctport->{CONFIG}->{"trunkmap"}}){
		$ctport->logger("make_external_call: Checking port $tmp ");
		if(check_bparty($tmp)){
			$ctport->logger("make_external_call: Found Free port!");
			$bparty = $tmp;
			$ctport->bridge($bparty);
			return "handle call";
		}
	}

	# no external lines available
	return "handle wait for hangup";
}

sub check_bparty($) {
	my ($bparty)=@_;
	my ($who, $event);

	# see if bparty is busy, if so it wont reply to our request
	$ctport->send_event($bparty, "incoming_call");
	$ctport->start_timer_async(2);

	$ctport->logger("check_bparty: checking b-party");
	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("check_bparty: event $event");

		# all OK
		if ($event eq "call_accepted") {
			$ctport->stop_timer();
			$ctport->logger("check_bparty: call accepted OK");
			return 1;
		}

		# chosen port busy
		if ($event eq "call_not_accepted") {
			$ctport->stop_timer();
			$ctport->logger("check_bparty: call NOT accepted");
			return 0;
		}

		# this station hangs up while waiting for response
		if ($event eq "hangup") {
			$ctport->logger("check_bparty: hangup");
			$ctport->stop_timer();
			$ctport->send_event($bparty, "bye");
			return 0;
		}

		# no response from bparty after time out, assume busy
		if ($event eq "timer") {
			$ctport->logger("check_bparty: timer fired");
			return 0;
		}
	}
}

sub handle_call() {
	my ($who,$event,$extra);

	$ctport->logger("handle_call: Entering Handle Call State");
	
	while(1) {

		($who,$event,$extra) = $ctport->wait_for_event(); 
	        $ctport->logger("handle_call: who: [$who]  event: [$event]");

		if ($event eq "hangup") {
			# this station hangs up
			$ctport->logger("handle_call: got a hangup");
			$next_event_handler = handle_hangup();
			return $next_event_handler;
		}
		elsif ($event eq "bparty"){
			$bparty=$extra;
			$ctport->logger("handle_call: bparty set to $bparty");
		}
		elsif (($event eq "bye") && ($who == $bparty)) {
			# bparty tells us to finish call

			$ctport->logger("handle_call: handle bye hold_party [$hold_party] bparty [$bparty]");

			if ($hold_party == -1) {
				# regular hangup
				$ctport->logger("handle_call: wait for hangup");
				return "handle wait for hangup";
			}
			else {
				
				# some one on hold, reconnect to them
				$ctport->unbridge($bparty);
				$bparty = $hold_party;
				$hold_party = -1;
				$ctport->send_event($bparty, "unhold");
				$ctport->logger("handle_call: from unhold after bye ");
				$ctport->bridge($bparty);
				return "handle call";
			}

		}
		elsif (($event eq "bye") && ($who == $hold_party)) {
			# hold_party hangs up while we are talking to bparty
			$hold_party = -1;
		}
		elsif ($event eq "flash") {
			$ctport->logger("handle_call: got a flash");
			$next_event_state = handle_flash();
			if (defined($next_event_state)) {
				return $next_event_state;
			}
		}
		elsif ($event eq "join") {
			$ctport->logger("handle_call: got a join");
			$bparty=$who;
		}
		elsif ($event eq "hold") {
			$ctport->logger("handle_call: got a hold");
			return "handle onhold";
		}
		elsif($event eq "dtmf"){
			$ctport->logger("handle_call: got a dtmf, ignoring");
			# At the moment we discard inband DTMFs, as if we have OpenPBXes
			# at both ends of the call they will both respond...
			$ctport->clear();
		}
		elsif(($event eq "ping") && ($who == $bparty)){
			$ctport->logger("handle_call: got a ping");
			# trunk asking us to confirm call is still alive
			$ctport->send_event($bparty, "ping");
		}
		else {
			$ctport->logger("handle_call: Unknown Event $event from $who");
		}
	}
}

sub handle_wait_for_hangup() {
	my ($handle,$event, $next_state);
	$ctport->play_busy_tone_async();

	$ctport->logger("handle_wait_for_hangup: Entering Wait For Hangup State");

	while(1) {
		($handle,$event) = $ctport->wait_for_event(); 
		$ctport->logger("handle_wait_for_hangup: event $event");

		if ($event eq "hangup") {
			$ctport->play_terminate();
			if (hold_party_exists()) {
				$ctport->send_event($hold_party, "bye");
			}
			return "handle idle";
		}
		elsif ($event eq "toneend") {
			# busy beeper has timed out, start again
			$ctport->play_busy_tone_async();
		}
		elsif ($event eq "flash") {
			# we might have some one on hold
			if (hold_party_exists()) {
				return handle_flash();
			}
		}
		elsif ($event eq "loop drop") {
			# Ignore it!
		}
		else {
			$ctport->logger("handle_wait_for_hangup: Unknown Event $event");
		}
	}
}

sub unhold($) {
	my $tmp = $_[0];
	
	# tell bparty to un-hold
	$ctport->send_event($tmp, "unhold");
	$ctport->start_timer_async(3);

	$ctport->logger("unhold: Waiting For Event");

	# now wait for response from bparty
	while(1) {
		$event = $ctport->wait_for_event(); 
		$ctport->logger("unhold: event $event");

		if ($event eq "hangup") {
			# we hangup while waiting
			$ctport->send_event($tmp, "bye");
			return "handle idle";
		}
		elsif ($event eq "unhold_ok") {
			# bparty acknowledged OK
			return "handle call";			    
		}
		elsif ($event eq "timer") {
			# time out waiting for unhold
			$ctport->send_event($tmp, "bye");
			return "handle idle";
		}
		else {
			$ctport->logger("unhold: Unknown Event $event");
		}
	}
}

sub star_menu() {
	my ($ret);

	$ctport->logger("star_menu: Entering Star Menu");

	while(1) {
		$ctport->clear;               # clears any events and digit buffer
		$ctport->play("$promptpath/star-menu1.wav");
		$ctport->ctsleep(5);          # Wait 5s unless event occurs
		
		if (defined($ctport->event)) {
			if ($ctport->event eq "1") {       
				$ctport->logger("star_menu: Menu 1 - Voicemail Menu");
				return "handle voicemail menu";
			}	
			if ($ctport->event eq "2") {
				$ctport->logger("star_menu: Menu 2 - Station Manager");
				# DR 14/8/03 disable for now, pending testing
				# return "handle station manager";
				return "handle wait for hangup";
			}
			# BK 26/8/03 This brakes the standard that * will
			# return you to the previous menu, as it does in all
			# other menus.
			if ($ctport->event eq "*") {
				$ctport->logger("star_menu: Menu * - Pick up ringing ext");
				return grab_ringing_extension();
			}
			if ($ctport->event eq "hangup") {
				return "handle idle";
			}
		}
	}
}

sub handle_hangup($) {
	my $blind = shift;
	
	$ctport->logger("handle_hangup: terminating play(s)");
	$ctport->play_terminate();
	$ctport->logger("handle_hangup: stopping the timer");
	$ctport->stop_timer();

	$ctport->logger("handle_hangup: checking for hold parties");
	if (!hold_party_exists()) {
		# no one on hold
		if (bparty_exists()) {
			# but we are in a call
			$ctport->logger("handle_hangup: no hold party, sending bye to $bparty");
			$ctport->send_event($bparty, "bye");
			$ctport->unbridge($bparty);
		}
	}
	else {
		# some one on hold
		if (bparty_exists()) {
			# we are in a call with a bparty

			# therefore connect bparty and hold party
			$ctport->unbridge($bparty);
#			if (($ctport->{CONFIG}{ports}{$bparty} eq "trunk")&&
#                            ($ctport->{CONFIG}{ports}{$hold_party} eq "trunk")){
#				$ctport->logger("handle_hangup: Both holdies are trunks! "
#						. "hanging up both!");
#				$ctport->send_event($bparty, "bye");
#				$ctport->send_event($hold_party, "bye");
#			}
#			else {
				if (defined($blind)) {
					# blind tfer
					$ctport->logger("handle_hangup: blind transfer");
					$ctport->send_message($bparty,("bparty",$hold_party));
					$ctport->send_message($hold_party,("bparty",$bparty));
					$ctport->send_event($hold_party, "unhold_blind");
				}
				else {
					$ctport->logger("handle_hangup: normal transfer bparty "
							. "$bparty hold_party $hold_party");
					$ctport->send_message($bparty,("bparty",$hold_party));
					$ctport->send_message($hold_party,("bparty",$bparty));
					$ctport->send_event($hold_party, "unhold");
					$ctport->join($bparty, $hold_party);
				}
#			}
		}
		else {
			# some one on hold, but no one to connect them to
			# therefore kill call
			$ctport->send_event($hold_party, "bye");
		}
	}

	# whatever happens, a hangup means we go back to idle state
	return "handle idle";
}

sub handle_flash() {
	my $tmp;

	if (!hold_party_exists()) {
		# no hold party - go to dial tone

		# make sure we are actually in a call

		if (bparty_exists()) {
                        # no current hold party, put bparty on hold and goto dial menu
			$ctport->stop_timer();
			$ctport->stop_tone();
 			$ctport->send_event($bparty, "hold");
			$ctport->unbridge($bparty);
			$hold_party = $bparty;
			$bparty = -1;
			return "handle dial menu";
		}

		# not in a call, and no hold party, do nothing
		return "";
	}
	else {
		# we already have some one on hold

		$ctport->logger("handle_flash: hold party exists");
		$ctport->play_terminate();
		if (bparty_exists()) {

			# flash therefore means swap bparty and hold party

			$ctport->logger("handle_flash: bparty exists, swapping");
			$ctport->send_event($bparty, "hold");
			$ctport->unbridge($bparty);

			$tmp = $hold_party;
			$hold_party = $bparty;
			$bparty = $tmp;
		
		}
		else {
			# no bparty, therefore reconnect hold party

			$ctport->logger("handle_flash: no bparty, connecting hold party");
			$bparty=$hold_party;
			$hold_party=-1;
		}

		$ctport->send_event($bparty, "unhold");
		$ctport->bridge($bparty);
		return "handle call";
	}
}

sub handle_onhold() {
	my ($moh, $who, $event,$mohidx, $extra);

	$ctport->logger("handle_onhold: Entering OnHold State");

	$mohidx=0;
	if (defined($MOH[$mohidx])) {
		$ctport->logger("handle_onhold: playing $MOH[$mohidx]");
		$ctport->play_async($MOH[$mohidx]);
		$mohidx++;
	}

	while(1) {
		my ($who,$event,$extra)= $ctport->wait_for_event(); 
		$ctport->logger("handle_onhold: event $event");

		if ($event eq "bye") {
			# other party disappears while we are on hold
			$ctport->play_stop();
			return "handle wait for hangup";
		}
		elsif ($event eq "hangup") {
			$ctport->play_stop();
			$ctport->send_event($bparty, "bye");
			return "handle idle";
		}
		elsif ($event eq "playend") {
			# iterate thru MOH files
			if (defined($MOH[$mohidx])){
				$ctport->play_async($MOH[$mohidx]);
				$mohidx++;
			}
			else {
				$mohidx=0;
				if (defined($MOH[$mohidx])){
					$ctport->play_async($MOH[$mohidx]);
					$mohidx++;
				}
			}
		}
		elsif ($event eq "unhold") {
			# tell bparty we are still about!
			$ctport->play_stop();
			$ctport->send_event($bparty, "unhold_ok");
			return "handle call";
		}
		elsif ($event eq "unhold_blind") {
			# jump to ring_internal_call state
			$ctport->play_stop();
			return "ring internal call";
		}
		elsif ($event eq "bparty") {
			# change bparties, e.g. for blind tfer
			$bparty = $extra;
			$ctport->logger("handle_onhold: new bparty $bparty");
		}
		else {
			$ctport->logger("handle_onhold: Unknown Event $event");
		}
	}
}

sub hold_party_exists() {
	if ($hold_party != -1) {
		return 1;
	}
	else {
		return 0;
	}
}

sub bparty_exists() {
	if ($bparty != -1) {
		return 1;
	}
	else {
		return 0;
	}
}

sub get_trunk($){
	my $arg=shift;
	my @trunks;
	if (defined $arg){
		@trunks=split /|/,$arg;
	}
	else {
		@trunks= keys %{$ctport->{CONFIG}->{"trunkmap"}};
	}
	foreach $tmp  (@trunks){
		$ctport->logger("make_external_call:[get_trunk] Checking port $tmp ");
		if(check_bparty($tmp)){
			$ctport->logger("make_external_call:[get_trunk] Found Free port!");
			return($tmp);
		}
	}
	return("-1");
}

sub generate_hook_flash() {

	# only valid if in station-trunk call, but we need to handle other cases

	# are we in a call, with bparty on hold?
	if  ($hold_party != -1) {
		# yes, then lets see if bparty is a trunk.
		if ($ctport->{CONFIG}{ports}{$hold_party} eq "trunk") {
			# OK, go ahead and generate external flash
			$ctport->send_event($hold_party,"generate flash");
			$bparty=$hold_party;
			$hold_party=-1;
			return "handle call";
		}
	}

	# otherwise do nothing

	return "";
}

sub get_lcr($){
	my $digits=shift;
	my $trunk="-1";
	my $diglen = length($digits);
	$ctport->logger("make_external_call:[LCR] Looking for a route for $digits length $diglen");
	foreach my $lcr (keys %{$ctport->{CONFIG}->{"lcr"}}){
		my $line=$ctport->{CONFIG}->{"lcr"}->{$lcr};
		$ctport->logger("make_external_call:[LCR] checking route: $lcr ");
		my ($match,$minlen,$maxlen,$remnum,$addnum,$trunks)=split/:/,$line;
		$ctport->logger("make_external_call:[LCR] checking route: $match,$minlen,$maxlen,$remnum,$addnum,$trunks");
		if (($match eq "*")||($digits=~/^$match.*/)){
		$ctport->logger("make_external_call:[LCR] checking route: matches on >$match<");
		$ctport->logger("make_external_call:[LCR] checking route: checking if $minlen <= $diglen <= $maxlen");
		if (($diglen>=$minlen)&&($diglen<=$maxlen)){
			$ctport->logger("make_external_call:[LCR] Matched on route for $digits");
			$digits=~s/^$remnum//;
			$digits=$addnum.$digits;
			$ctport->logger("make_external_call:[LCR] Digits are now $digits");
			if ($trunks eq "*"){
				$trunk=get_trunk(undef);
			}
			else {
				$trunk=get_trunk($trunks);
			}
			return($trunk,$digits);
		}
		}
	}
	if ($trunk eq "-1"){
		$trunk=get_trunk(undef);
		return($trunk,$digits);
	}
}


sub station_manager(){
	my ($dialplan,$pin,$email,$vmail,$profileid,$profile);
	my $result="";
	# find out if we have a profile associated
	foreach my $bit (keys %{$ctport->{CONFIG}{"profiles"}}){
		$profile=$ctport->{CONFIG}{"profiles"}{$bit};
		($dialplan,$pin,$email,$vmail)=split /,/,$profile;
		foreach my $dp (split /\:/,$dialplan){
			if ($dp eq $aparty){
				$profileid=$bit;
			}
		}
	}
	if (!defined $profileid){
	$result=$ctport->play("audio/statman1.wav $aparty audio/statman2.wav audio/statman4.wav");
	}
	else {
		my $num=$ctport->number($profileid);
		$result=$ctport->play("audio/statman1.wav audio/statman3.wav $num audio/statman4.wav");
	}
	if ($result eq "EVENT"){
		return "handle wait for hangup";
	}
	my $i=0;
	while ($i<20){
		my $digits=$ctport->collect(1,5,5);
		if ($digits eq "1"){
			if(defined $profileid){
				$result=$ctport->play("audio/statman8.wav");
				if ($result eq "EVENT"){
					return "handle wait for hangup";
				}
			}
			else {
				my $result = login_profile();
				return($result);
			}
		}
		elsif($digits eq "2"){
			my $result = logout_profile($profileid);
			return($result);
		}
		elsif($digits eq "*"){
			return "handle star menu";
		}
		elsif($digits eq "EVENT"){
			return "handle wait for hangup";
		}
		$i++;
		$result=$ctport->play("audio/statman4.wav");
		if ($result eq "EVENT"){
			return "handle wait for hangup";
		}
	}
	return "handle wait for hangup";
}

sub login_profile(){
	my ($ext,$ipin,$profile);
	my ($dialplan,$pin,$email,$vmail);
	my $i=0;
	my $result="";
	while($i<20){
		$result=$ctport->play("audio/statman5.wav"); #please enter your extension number.
		if ($result eq "EVENT"){
			return "handle wait for hangup";
		}
		$ext=$ctport->collect(3,15,5);
		if ($ext eq "*"){
			return "handle star menu";
		}
		elsif ($ext eq "EVENT"){
			return "handle wait for hangup";
		}
		$profile=$ctport->{CONFIG}{"profiles"}{$ext};
		if (defined $profile){
			($dialplan,$pin,$email,$vmail)=split /,/,$profile;
			$result=$ctport->play("audio/statman6.wav"); #please enter your pin number.
			if ($result eq "EVENT"){
				return "handle wait for hangup";
			}
			$ipin=$ctport->collect(4,20,5);
			if ($ipin eq "*"){
				return "handle star menu";
			}
			elsif ($ipin eq "EVENT"){
				return "handle wait for hangup";
			}
			elsif ($ipin eq $pin){
				$dialplan =~ s/-1/$aparty/;
				save_profile($ext,$dialplan,$pin,$email,$vmail);
				$ctport->play("audio/statman6b.wav"); #Thank you, you are now logged in.
				return "handle star menu";
			}
			$result=$ctport->play("audio/statman6a.wav"); #That extesion and pin combination is incorrect.
			if ($result eq "EVENT"){
				return "handle wait for hangup";
			}
		}
		else {
			$result=$ctport->play("audio/statman5a.wav"); #That extension doesnt exist.
			if ($result eq "EVENT"){
				return "handle wait for hangup";
			}
		}
		$i++;
	}
	return "handle wait for hangup";
}

sub logout_profile($){
	my $profileid=shift;
	my ($dialplan,$pin,$email,$vmail);
	$profile=$ctport->{CONFIG}{"profiles"}{$profileid};
	($dialplan,$pin,$email,$vmail)=split /,/,$profile;
	my $i=0;
	my $result="";
	while ($i<20){
		$result=$ctport->play("audio/statman6.wav"); #please enter your pin number.
		if ($result eq "EVENT"){
			return "handle wait for hangup";
		}
		$ipin=$ctport->collect(4,20,5);
		if ($ipin eq "*"){
			return "handle star menu";
		}
		elsif ($ipin eq "EVENT"){
			return "handle wait for hangup";
		}
		elsif ($ipin eq $pin){
			my $y=0;
			my $confirm=-1;
			while (($y<10)&&($confirm ne "1")){
				$result=$ctport->play("audio/statman7.wav"); #press 1 to confirm you wish to log out this profile.
				if ($result eq "EVENT"){
					return "handle wait for hangup";
				}
				$confirm=$ctport->collect(1,10,10);
				if ($confirm eq "*"){
					return "handle star menu";
				}
				elsif ($confirm eq "EVENT"){
					return "handle wait for hangup";
				}
				$y++;
			}
			if ($y<10){
				$dialplan =~ s/$aparty/-1/;
				save_profile($profileid,$dialplan,$pin,$email,$vmail);
				$ctport->play("audio/statman6b.wav"); #Thank you, you are now logged out.
				return "handle star menu";
			}
		}
		$result=$ctport->play("audio/statman6a.wav"); #That extesion and pin combination is incorrect.
		if ($result eq "EVENT"){
			return "handle wait for hangup";
		}
		$i++;
	}
	return "handle wait for hangup";
}

sub grab_ringing_extension() {
	my $tmp;

	$ctport->logger("grab_ringing_extension: Entering Grab Ringing Extension");

	# see if we can work out which trunk is ringing

	foreach $tmp (keys %{$ctport->{CONFIG}->{"trunkmap"}}){
		$ctport->logger("grab_ringing_extension: Checking trunk $tmp ");
		if(is_trunk_ringing($tmp)){
			$ctport->logger("grab_ringing_extension: Found ringing trunk!");
			$bparty = $tmp;
			$ctport->send_event($bparty,"pickup");
			return "handle call";
		}
	}

	# no external lines availableringing trunks found
	return "handle wait for hangup";
}

sub is_trunk_ringing($) {
	my ($trunk)=@_;
	my ($who, $event);

	$ctport->send_event($trunk, "are u ringing");
	$ctport->start_timer_async(1);

	$ctport->logger("is_trunk_ringing: checking $trunk");
	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("is_trunk_ringing: event $event");

		if ($event eq "yes") {
			$ctport->stop_timer();
			$ctport->logger("is_trunk_ringing: $trunk ringing");
			return 1;
		}

		if ($event eq "no") {
			$ctport->stop_timer();
			$ctport->logger("is_trunk_ringing: $trunk NOT ringing");
			return 0;
		}

		# this station hangs up while waiting for response
		if ($event eq "hangup") {
			$ctport->logger("is_trunk_ringing: hangup");
			$ctport->stop_timer();
			return 0;
		}

		# no response from bparty after time out, assume busy
		if ($event eq "timer") {
			$ctport->logger("is_trunk_ringing: timer fired");
			return 0;
		}
	}
}

