#!/bin/bash
# test002.sh
# David Rowe 3/4/03
#
# Automated openpbx test startup script
#
HOME=`pwd`;
SERVER_HOME=${HOME}/../../src
SERVER=ctserver
DESC="openpbx test002"
SETCOLOR_NORMAL="echo -en \\033[0;39m"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"

test -f $SERVER_HOME/$SERVER || echo "Error: Can't find $SERVER"
set -e
case "$1" in
  start)
	# check it's not already running

	if [ "$(/sbin/pidof $SERVER)" == "" ] ; then
	    echo -n "Starting $DESC (please wait 10 secs): "

	    # start ctserver
	    $SERVER_HOME/$SERVER -d -nv --log /tmp
	    sleep 10

	    # check that it started OK

	    if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
		$SETCOLOR_FAILURE
		echo "[  FAILED  ]"
		$SETCOLOR_NORMAL
		echo "Error - Try running $SERVER_HOME/$SERVER from console" 
	    else
		$SETCOLOR_SUCCESS
		echo "[  OK  ]"
		$SETCOLOR_NORMAL
	    fi
	fi

	echo -n "Starting PBX scripts: "
	pushd ../.
	./trunk.pl -d -p 9 -c tests/test002.conf
	./station.pl -d -p 0 -c tests/test002.conf
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	popd

	echo -n "Starting simulated phone script: "
	perl simphone_out.pl -d
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  stop)
	echo -n "Stopping $DESC: "

	# see if server is running

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - $SERVER is not running!"
	    exit 1
	fi

	# OK, lets kill it

	cat /var/run/ctserver.pid | xargs kill
	sleep 1

	# check that it was killed

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_SUCCESS
	    echo "[  OK  ]"
	    $SETCOLOR_NORMAL
	else
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - couldn't kill $SERVER!"
	fi
	;;
  stop-perl)
	echo -n "Stopping perl scripts (keep ctserver running): "

	ps -deafwww | grep "perl simphone_out.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL
	ps -deafwww | grep "perl -w ../trunk.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL
	ps -deafwww | grep "perl -w ../station.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL

	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  *)
	echo "Usage: test002.sh {start|stop|stop-perl}"
echo "
# Tests a station-trunk out going call
#   - simulated phone takes line off hook,
#   - waits for dial tone, dials 0
#   - waits a few seconds then hangs up and starts again
#   - simulated phone connect to station port
#   - nothing needed to simulate line
#
# One OS12 required, connect a RJ45 ethernet cable 
# between J1 and J3 (top and bottom).
# Configure ports 0..7 as station, ports 8..11 as trunk
# (loop start).
# 
# port 9, trunk.pl, single trunk port
# port 0, station.pl, single station port
# port 8, simphone_out.pl, simulates phone on station port
# 
# PASS: Check the log for simphone_out in /tmp to confirm that
#       less than 1 in 1000 failed.
"
	exit 1
	;;
esac

exit 0
