#!/bin/bash
#set -x
# test003.sh
# David Rowe 3/4/03
#
# Automated openpbx test startup script
#
HOME=`pwd`
SERVER_HOME=${HOME}/../../src
SERVER=ctserver
DESC="openpbx test003"
SETCOLOR_NORMAL="echo -en \\033[0;39m"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"

test -f $SERVER_HOME/$SERVER || echo "Error: Can't find $SERVER"
set -e
case "$1" in
  start)
	# check it's not already running

	if [ "$(/sbin/pidof $SERVER)" == "" ] ; then
	    echo -n "Starting $DESC (please wait 10 secs): "

	    # start ctserver
	    $SERVER_HOME/$SERVER -d -nv --log /tmp
	    sleep 10

	    # check that it started OK

	    if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
		$SETCOLOR_FAILURE
		echo "[  FAILED  ]"
		$SETCOLOR_NORMAL
		echo "Error - Try running $SERVER_HOME/$SERVER from console" 
	    else
		$SETCOLOR_SUCCESS
		echo "[  OK  ]"
		$SETCOLOR_NORMAL
	    fi
	fi

	echo -n "Starting PBX scripts: "
	pushd ../.
	./station.pl -d -p 0 -c tests/test003.conf
	./station.pl -d -p 1 -c tests/test003.conf
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	popd

	echo -n "Starting simulated phone script: "
	perl simphone.pl -d
	perl simphone_ext.pl -d
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  stop)
	echo -n "Stopping $DESC: "

	# see if server is running

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - $SERVER is not running!"
	    exit 1
	fi

	# OK, lets kill it

	cat /var/run/ctserver.pid | xargs kill
	sleep 1

	# check that it was killed

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_SUCCESS
	    echo "[  OK  ]"
	    $SETCOLOR_NORMAL
	else
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - couldn't kill $SERVER!"
	fi
	;;
  stop-perl)
	echo -n "Stopping perl scripts (keep ctserver running): "

	ps -deafwww | grep "perl simphone_out.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL
	ps -deafwww | grep "perl -w ../trunk.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL
	ps -deafwww | grep "perl -w ../station.pl" | grep -v "grep" | awk '{print $2}' | xargs kill -KILL

	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  *)
	echo "Usage: test003.sh {start|stop|stop-perl}"
echo "
# Tests a station-station call
#   - simulated phone takes line off hook,
#   - waits for dial tone, dials 1
#   - waits a few seconds then hangs up and starts again
#   - simulated phone connect to station port
#   - nothing needed to simulate line
# One OS12 required, connect a RJ45 ethernet cable 
# between J1 and J3 (top and bottom).
# Configure ports 0..7 as station, ports 8..11 as trunk
# (loop start).
# 
# port 0, station.pl, single station port
# port 1, station.pl, single station port
# port 8, simphone.pl, simulates phone on station port
# port 9, simphone_ext.pl, simulates phone on station port
#
# PASS: Check that the number of calls received is with in 1 of the
#       the number of calls made. The information is in the simphone
#       and simphone_ext log files in /tmp
"

	exit 1
	;;
esac

exit 0
