#!/bin/bash
# test005.sh
# Ripped of from David Rowe by Ben Kramer on the 8/4/03
#
# Automated openpbx test startup script
#

HOME=`pwd`;
SERVER_HOME=${HOME}/../../src
SERVER=ctserver
DESC="transfer testing"
SETCOLOR_NORMAL="echo -en \\033[0;39m"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"

test -f $SERVER_HOME/$SERVER || echo "Error: Can't find $SERVER"
set -e
case "$1" in
  start)
	# check it's not already running

	if [ "$(/sbin/pidof $SERVER)" == "" ] ; then
	    echo -n "Starting $DESC (please wait 10 secs): "

	    # start ctserver
	    $SERVER_HOME/$SERVER -d -nv --log /tmp
	    sleep 10

	    # check that it started OK

	    if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
		$SETCOLOR_FAILURE
		echo "[  FAILED  ]"
		$SETCOLOR_NORMAL
		echo "Error - Try running $SERVER_HOME/$SERVER from console" 
	    else
		$SETCOLOR_SUCCESS
		echo "[  OK  ]"
		$SETCOLOR_NORMAL
	    fi
	fi

	echo -n "Starting PBX scripts: "
	pushd ../.
	./trunk.pl -d -p 11 -c tests/test004.conf
	./station.pl -d -p 0 -c tests/test004.conf
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	popd

	echo -n "Starting simulated line & phone script: "
	perl test005callgen.pl -d
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  stop)
	echo -n "Stopping $DESC: "

	# see if server is running

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - $SERVER is not running!"
	    exit 1
	fi

	# OK, lets kill it

	cat /var/run/ctserver.pid | xargs kill
	sleep 1

	# check that it was killed

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_SUCCESS
	    echo "[  OK  ]"
	    $SETCOLOR_NORMAL
	else
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - couldn't kill $SERVER!"
	fi
	;;
  stop-perl)
  
        echo "Not Implemented"
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  *)
	echo "Usage: $0 {start|stop|stop-perl}"
echo "
# Tests audio path by playing digits.
#
# One OS12 required, connect a RJ45 ethernet cable 
# between J2 and J3 (middle and closed to motherboard).
# Configure ports 0..7 as station, ports 8..11 as trunk
# (loop start).
# 
# port 11, trunk.pl, single trunk port
# port 0, station.pl, station port A 
# ports 3, 8,  test005callgen.pl, simulates incoming
# lines and simphone_out.pl, simulates phone on station port
#
# PASS: less than 1 in 1000 calls fail. This can be checked
#       from the test005 log in /tmp
"
	exit 1
	;;
esac

exit 0
