#!/usr/bin/perl -w
# test004callgen.pl
# David Rowe 7/4/03
#
# Handles the call gen a station phone logic for testing
# transfers.  Simulates one external line and two 
# station phones

BEGIN {
  push(@INC, "../../CTPort");
};

use CTPort;
use POSIX;
use IO::Handle;

sub procargs(@);
sub wait_for_ring_simphoneB();

my $its = 0;
my $btimeout = 0;
my $daemon = 0;
my $simline;
my $simphoneA;
my $simphoneB;
my $dialstr="1,2,3,4,5,6";

procargs(@ARGV);

$simline = new Telephony::CTPort(1203);
$simphoneA = new Telephony::CTPort(1208);

if ($daemon) {
	$simline->daemonize($daemon);
	my $logfile="/tmp/test005.log";
	$simline->openlogger($logfile);
}
$simphoneA->on_hook();    


while(1) {
	$simline->start_ring_async();
	$simline->logger("ext line ringing, waiting for station A to ring");
	$simphoneA->wait_for_ring();
	$simphoneA->off_hook();
	$simline->logger("station A answered");
	$its++;
	sleep(1);
	$simline->dial($dialstr);
	$simline->logger("dialed [$dialstr]");
	my $colstr = $simphoneA->collect(6,20,2);
	if ($colstr eq "123456"){
		$digs++;
	}
	else {
		$simline->logger("Got [$colstr] not \"123456\"");
	}

	$simline->logger("calls answered: $its All digits collected: $digs");
	$simphoneA->on_hook();    
	sleep(2);
}

# this sub used to time out if we dont get ring after 10 seconds
# due to very rare case when DTMF gets missed

sub wait_for_ring_simphoneB() {

	$simphoneB->start_timer(10);
	while (1) {
		($handle, $event) = $simphoneB->wait_for_event();
		$simline->logger("[$handle] $event");

		if ($event eq "ring") {
			return;
		}
		if ($event eq "timer") {
			$simline->logger("TIMEOUT: NO RING ON STATION B");
			$btimeout++;
			return;
		}
	}
}

sub procargs(@){
	my $bit;

	while($bit = shift @ARGV){
		if ($bit =~ /-d/){
			$daemon = 1;
		}
	}
}

