#!/usr/bin/perl -w
# testm2001callgen.pl
#
#	./station.pl -d -p 0 -c tests/testm2001.conf
#	./station.pl -d -p 1 -c tests/testm2001.conf
#	./trunk.pl -d -p 11 -c tests/testm2001.conf
#       + leaves many messages
#       + listens to messages and randomly deletes
#       + messages may be listened and left simultaneusly
#       + messages may come from station and trunks
#	     + messages could be left because:
#		   + busy
#		   + ringout
#		   + directed straight to vmail
#       + should hangup while:
#	     + leaving message
#		 + during prompt

BEGIN {
  push(@INC, "../../CTPort");
};

use CTPort;
use POSIX;
use IO::Handle;
use Cwd qw(cwd);

my $vmaildir = cwd."/../vmail/bkramer";

sub procargs(@);
sub leave_msg($);

my $daemon = 0;
my $simtrunk;
my $simstat;
my $collector;
my $totmsgs=0;
my $collmsgs=0;

procargs(@ARGV);

$collector = new Telephony::CTPort(1208);
$simstat = new Telephony::CTPort(1209);
$simtrunk = new Telephony::CTPort(1203);

$simstat->on_hook();
$collector->on_hook();

if ($daemon) {
	$collector->daemonize($daemon);
	my $logfile="/tmp/testm2001callgen";
	$collector->openlogger($logfile);
}
if (opendir(DIR,"$vmaildir")){
	@wavs= grep {/.wav|!^./} readdir(DIR);
	closedir(DIR);
	my $num=$#wavs;
	$collector->logger("Cleaning out $num old files");
	foreach $file (@wavs){
		unlink("$vmaildir/$file");
	}
}
else {
	$collector->logger("Couldnt open $vmaildir");
}

while(1) {
	# Leave a message trunk->ringout
	$collector->logger("Leaving message trunk->ringout");
	# ring collector
	$simtrunk->start_ring_async();
	$waiting=1;
	while ($waiting){
		($handle,$event)=$simtrunk->wait_for_event();
		if ($event eq "pickup"){
			$collector->logger("Got pickup");
			$waiting=0;
		}
		else {
			$collector->logger("Got [$event] while waiting for pickup");
		}
	}
	# wait for greeting to play
	sleep(5);
	$simtrunk->clear();
	# leave a message
	leave_msg($simtrunk);
	# wait for hangup
	$simtrunk->play_busy_tone_async();
	$waiting=1;
	while($waiting){
		($handle,$event)=$simtrunk->wait_for_event();
		if ($event eq "hangup"){
			$collector->logger("Got hangup");
			$waiting=0;
		}
		elsif ($event eq "toneend"){
			$collector->logger("Got [$event] while waiting for hangup");
			$simtrunk->play_busy_tone_async();
		}
		else {
			$collector->logger("Got [$event] while waiting for hangup");
		}
	}
	$simtrunk->play_terminate();
	$totmsgs++;
	sleep(1);

	# Leave a message trunk->busy
	$collector->logger("Leaving message trunk->busy");
	# put collector off hook
	$collector->off_hook();
	# ring collector
	$simtrunk->start_ring_async();
	$waiting=1;
	while ($waiting){
		($handle,$event)=$simtrunk->wait_for_event();
		if ($event eq "pickup"){
			$collector->logger("Got pickup");
			$waiting=0;
		}
		else {
			$collector->logger("Got [$event] while waiting for pickup");
		}
	}
	# wait for greeting to play
	sleep(5);
	$simtrunk->clear();
	# leave a message
	leave_msg($simtrunk);
	$simtrunk->dial("#");
	# wait for hangup
	$waiting=1;
	while($waiting){
		($handle,$event)=$simtrunk->wait_for_event();
		if ($event eq "hangup"){
			$collector->logger("Got hangup");
			$waiting=0;
		}
		else {
			$collector->logger("Got [$event] while waiting for hangup");
		}
	}
	$totmsgs++;
	# put collector on hook
	$collector->on_hook();
	sleep(1);

	# leave message stat->ringout
	$collector->logger("Leaving message stat->ringout");
	$simstat->off_hook();
	$simstat->dial("100");
	#$simstat->wait_for_ring(2);
	# wait for greeting to play
	sleep(5);
	# leave a message
	leave_msg($simstat);
	$simstat->dial("#");
	$simstat->on_hook();
	$totmsgs++;
	sleep(1);

	# leave message stat->busy
	$collector->logger("Leaving message stat->busy");
	$collector->off_hook();
	$simstat->off_hook();
	$simstat->dial("100");
	# wait for greeting to play
	sleep(5);
	# leave a message
	leave_msg($simstat);
	$simstat->dial("#");
	$totmsgs++;
	$simstat->on_hook();
	$collector->on_hook();

	# confirm numer of messages on machine is the same as left
	$collector->logger("Checking $vmaildir for vmails");
	if (opendir(DIR,"$vmaildir")){
		@wavs= grep {/.wav|!^./} readdir(DIR);
		closedir(DIR);
	}
	else {
		$collector->logger("Couldnt open $vmaildir");
	}
	$numwavs=$#wavs + 1;
	$collector->logger("Found [$numwavs] msgs out of total [$totmsgs] minus collected [$collmsgs]");
	# collect random number of messages
	$thismany=int(rand($numwavs));
	$collector->logger("Going to collect [$thismany] vmails...");
	$collector->off_hook();
	$collector->dial("*11");
	$collector->logger("listening to instructions...");
	sleep(5);
	$collector->logger("listening to first message");
	while ($thismany-- > 0){
		sleep(3);
		$collector->dial("2");
		$collector->logger("deleted message...");
		$collmsgs++;
	}
	$collector->on_hook();
	# confirm numer of messages on machine is the same as left
	undef @wavs;
	$collector->logger("Checking $vmaildir for vmails");
	if (opendir(DIR,"$vmaildir")){
	@wavs= grep {/.wav|!^./} readdir(DIR);
	closedir(DIR);
	}
	else {
		$collector->logger("Couldnt open $vmaildir");
	}
	$numwavs=$#wavs + 1;
	$collector->logger("Found [$numwavs] msgs out of total [$totmsgs] minus collected [$collmsgs]");

	# report results

	$collector->logger("Take a short nap");
	sleep(2)
}

sub leave_msg($){
	my $line=shift;
	$line->play("1 2 3 4 5 6 7 8 9 10");
	$line->play("1 2 3 4 5 6 7 8 9 10");
}

sub procargs(@){
	my $bit;
	while($bit = shift @ARGV){
		if ($bit =~ /-d/){
			$daemon = 1;
		}
	}
}

