#!/bin/bash
# testm2002.sh
#
# Automated openpbx test startup script
#

HOME=`pwd`
SERVER_HOME=${HOME}/../../src
SERVER=ctserver
DESC="testm2002"
SETCOLOR_NORMAL="echo -en \\033[0;39m"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"

test -f $SERVER_HOME/$SERVER || echo "Error: Can't find $SERVER"
set -e
case "$1" in
  start)
	# check it's not already running

	if [ "$(/sbin/pidof $SERVER)" == "" ] ; then
	    echo -n "Starting $DESC (please wait 10 secs): "

	    # start ctserver
	    $SERVER_HOME/$SERVER -d -nv --log /tmp
	    sleep 10

	    # check that it started OK

	    if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
		$SETCOLOR_FAILURE
		echo "[  FAILED  ]"
		$SETCOLOR_NORMAL
		echo "Error - Try running $SERVER_HOME/$SERVER from console" 
	    else
		$SETCOLOR_SUCCESS
		echo "[  OK  ]"
		$SETCOLOR_NORMAL
	    fi
	fi

	echo -n "Starting PBX scripts: "
	pushd ../.
	./station.pl -d -p 0 -c tests/testm2002.conf
	./station.pl -d -p 1 -c tests/testm2002.conf
	./station.pl -d -p 2 -c tests/testm2002.conf
	./station.pl -d -p 3 -c tests/testm2002.conf
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	popd

	echo -n "Starting simulated line & phone scripts: "
	perl testm2002callgen.pl -d -p 8
	perl testm2002callgen.pl -d -p 9
	perl testm2002callgen.pl -d -p 10
	perl testm2002callgen.pl -d -p 11
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  stop)
	echo -n "Stopping $DESC: "

	# see if server is running

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - $SERVER is not running!"
	    exit 1
	fi

	# OK, lets kill it
	kill -TERM `cat /var/run/ctserver.pid` 2>/dev/null
	CTSVR=`ps -deaf | grep "ctserver -d" | grep -v "grep" | awk '{print $2}'`
	if [ -n "$CTSVR" ];then
		echo "ctserver could not be stopped, trying harder..."
		echo $CTSVR|xargs -n1 kill -9
		sleep 3
	fi

	sleep 1

	# check that it was killed

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_SUCCESS
	    echo "[  OK  ]"
	    $SETCOLOR_NORMAL
	else
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - couldn't kill $SERVER!"
	fi
	;;
  stop-perl)
  
        echo "Not Implemented"
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  *)
	echo "Usage: $0 {start|stop|stop-perl}"
echo "
# Tests star menu
#
# One OS12 required, connect a RJ45 ethernet cable 
# between J1 and J3 (top and bottom).
# Configure ports 0..7 as station, ports 8..11 as trunk
# (loop start).
# 
"
	exit 1
	;;
esac

exit 0
