#!/usr/bin/perl -w
# testm2003callgen.pl
# David Rowe 7/4/03
#
# Handles the call gen a station phone logic for testing
# transfers.  Simulates one external line and two 
# station phones

BEGIN {
  push(@INC, "../../CTPort");
};

use CTPort;
use POSIX;
use IO::Handle;

sub procargs(@);
sub wait_for_ring_simphoneB();

my $its = 0;
my $btimeout = 0;
my $daemon = 0;
my $simline;
my $simphoneA;
my $simphoneB;

procargs(@ARGV);

$simline = new Telephony::CTPort(1203);
$simphoneA = new Telephony::CTPort(1208);
$simphoneB = new Telephony::CTPort(1209);

if ($daemon) {
	$simline->daemonize($daemon);
	my $logfile="/tmp/testm2004";
	$simline->openlogger($logfile);
}
$simphoneA->on_hook();    
$simphoneB->on_hook();    

while(1) {
	$simline->start_ring_async();
	$simline->logger("ext line ringing, waiting for station A to ring");
	$simphoneA->wait_for_ring();
	$simline->logger("station A is ringing");
	sleep(1);

	# pickup ringing extension
	$simphoneB->off_hook();
	$simphoneB->dial(',**');
	$simline->logger("station B grabed line");

	$simphoneA->off_hook();
	$simphoneA->wait_for_dial_tone();
	$simline->logger("station A got dialtone, confirming B has line");
	$simphoneA->on_hook();
	sleep(2);
	$simphoneB->on_hook();
	$simline->logger("station B back on hook, finished");
	$its++;	
	$simline->logger("calls grabbed [$its]");

}

# this sub used to time out if we dont get ring after 10 seconds
# due to very rare case when DTMF gets missed

sub wait_for_ring_simphoneB() {

	$simphoneB->start_timer_async(10);
	while (1) {
		($handle, $event) = $simphoneB->wait_for_event();
		$simline->logger("[$handle] $event");

		if ($event eq "ring") {
			$simphoneB->stop_timer();
			return;
		}
		if ($event eq "timer") {
			$simline->logger("TIMEOUT: NO RING ON STATION B");
			$btimeout++;
			return;
		}
	}
}

sub procargs(@){
	my $bit;

	while($bit = shift @ARGV){
		if ($bit =~ /-d/){
			$daemon = 1;
		}
	}
}

