#!/bin/bash
# testm2005.sh
#
# Automated openpbx test startup script
#
HOME=`pwd`;
SERVER_HOME=${HOME}/../../src
SERVER=ctserver
DESC="testing"
SETCOLOR_NORMAL="echo -en \\033[0;39m"
SETCOLOR_SUCCESS="echo -en \\033[1;32m"
SETCOLOR_FAILURE="echo -en \\033[1;31m"

test -f $SERVER_HOME/$SERVER || echo "Error: Can't find $SERVER"
set -e
case "$1" in
  start)
	# check it's not already running

	if [ "$(/sbin/pidof $SERVER)" == "" ] ; then
	    echo -n "Starting $DESC (please wait 10 secs): "

	    # start ctserver
	    $SERVER_HOME/$SERVER -d -nv --log /tmp
	    sleep 10

	    # check that it started OK

	    if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
		$SETCOLOR_FAILURE
		echo "[  FAILED  ]"
		$SETCOLOR_NORMAL
		echo "Error - Try running $SERVER_HOME/$SERVER from console" 
	    else
		$SETCOLOR_SUCCESS
		echo "[  OK  ]"
		$SETCOLOR_NORMAL
	    fi
	fi

	echo -n "Starting PBX scripts: "
	pushd ../.
	./trunk.pl -d -p 11 -c tests/testm2004.conf
	./station.pl -d -p 0 -c tests/testm2004.conf
	./station.pl -d -p 1 -c tests/testm2004.conf
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	popd

	echo -n "Starting simulated line & phone script: "
	perl testm2004callgen.pl -d
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  stop)
	echo -n "Stopping $DESC: "

	# see if server is running

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - $SERVER is not running!"
	    exit 1
	fi

	# OK, lets kill it

	cat /var/run/ctserver.pid | xargs kill
	sleep 1

	# check that it was killed

	if [ "$(/sbin/pidof $SERVER)" = "" ] ; then
	    $SETCOLOR_SUCCESS
	    echo "[  OK  ]"
	    $SETCOLOR_NORMAL
	else
	    $SETCOLOR_FAILURE
	    echo "[  FAILED  ]"
	    $SETCOLOR_NORMAL
	    echo "Error - couldn't kill $SERVER!"
	fi
	;;
  stop-perl)
  
        echo "Not Implemented"
	$SETCOLOR_SUCCESS
	echo "[  OK  ]"
	$SETCOLOR_NORMAL
	;;
  *)
	echo "Usage: $0 {start|stop|stop-perl}"
echo "
This tests requires two machines, back to back. One machine
will be running the OpenPBX and the other will be running
this script.
You will have to change the jumper settings on the card used
by this script to be 8 trunk and 4 stations. Then the machines
can be connected by from port to port with regular ethernet 
cables.
This will use a combination of scripts testing:
 + leaving voicemail from trunk and station (2t2s ports)
 + retreiving voicemail (2t ports)
 + transfers (1s2t ports)
 + grabbing ringing extension (1s 2t ports)

Port  OpenPBX machine         CallGen machine
[0]   station.pl              leave-msg
[1]   station.pl              leave-msg
[2]   station.pl              get-vmail
[3]   station.pl              get-vmail
[4]   station.pl              transfer
[5]   station.pl              transfer
[6]   station.pl              grabber
[7]   station.pl              grabber
[8]   trunk.pl                leave-msg
[9]   trunk.pl                leave-msg
[10]  trunk.pl                transfer
[11]  trunk.pl                grabber

IMPORTANT: For the voicemail testing to work, you must mount
the vmail directory from the OpenPBX machine to the CallGen
machine on "/mnt/openpbxvmail"
"

	exit 1
	;;
esac

exit 0
