#!/usr/bin/perl -w
# trunk.pl
#
# ctserver script for PBX trunk port

use Telephony::CTPort;
use IO::Handle;
use Data::Dumper;
use Cwd qw(cwd);
use common;

# function prototypes ------------------------------------

sub handle_outgoing_call();
sub main_loop();
sub handle_onhold();
sub port_busy();
sub def_are_u_ringing();
sub ring_external_call($$);
sub handle_blind_transfer();

# constants --------------------------------------------------------

our $port=1208;
our $daemon=0;
our $station_ring_pause = 3;
our $configfile="pbx.conf";

# globals ----------------------------------------------------------

our $ctport;
our $aparty;
our $bparty=-1;              # handle of bparty
our $group;                  # group to work thru for calls and tfers
our $offhook_status=0;       # 0=> on-hook 1=> off-hook
our $promptpath;             # path to all audio prompts
our $vmailgreetpath;         # path to voice mail greetings
our $vmailmailpath;          # path to actual mail messages

# start code here -------------------------------------------------------


procargs(@ARGV);

$aparty=$port - 1200;
$ctport = new Telephony::CTPort($port);
$ctport->getconf($configfile);

my $moh_path = $ctport->{CONFIG}{"moh"}{"path"};
if (defined($moh_path)) {
	$ctport->set_paths($moh_path);
}
my @MOH;
foreach my $bit (keys %{$ctport->{CONFIG}{"moh"}}){
	push @MOH, $ctport->{CONFIG}{"moh"}{$bit};
}
foreach my $bit (keys %{$ctport->{CONFIG}{"dialmap"}}){
	my $port=$ctport->{CONFIG}{"dialmap"}{$bit};
	$ctport->{CONFIG}{"dialmap-rev"}{$port}=$bit;
}

# read in vaious paths

$vmailgreetpath = $ctport->{CONFIG}{"voicemail"}{"greetpath"};
$vmailmailpath = $ctport->{CONFIG}{"voicemail"}{"mailpath"};
$promptpath = $ctport->{CONFIG}{"general"}{"path"};

# daemonize after conf file read as path gets changed
if ($daemon) {
	my $logfile=cwd()."/log/$port-trunk";
	$ctport->openlogger($logfile);
	$ctport->daemonize($daemon);
}

$SIG{HUP}=\&sig_hup;

$ctport->set_event_handler("rolllog",\&roll_log);
$ctport->set_event_handler("load_config",\&load_config);
$ctport->set_event_handler("are u ringing", \&are_u_ringing);

$ctport->logger("STARTUP: Connected to CTServer on port $port.");
main_loop();

exit;

sub main_loop(){
	my $event_handler = "handle idle";
	$ctport->clear();

	while(1) {
		$ctport->setscript($event_handler);
		if ($event_handler eq  "handle idle") {
			$ctport->on_hook();
			$offhook_status=0;
			$event_handler = handle_idle();
		}
		elsif ($event_handler eq  "handle incoming call") {
			$event_handler = handle_incoming_call();
		}
		elsif ($event_handler eq  "handle outgoing call") {
			$event_handler = handle_outgoing_call();
		}
		elsif ($event_handler eq  "handle call") {
			$event_handler = handle_call();
		}
		elsif ($event_handler eq  "handle onhold") {
			$event_handler = handle_onhold();
		}
		elsif ($event_handler eq  "handle blind transfer") {
			$event_handler = handle_blind_transfer();
		}
		elsif ($event_handler eq  "handle wait for hangup") {
			$ctport->on_hook();
			$event_handler = "handle idle";
		}
		elsif ($event_handler eq  "handle group call") {
			$event_handler = handle_incoming_call(1);
		}
		elsif ($event_handler eq  "handle autoattendant answer") {
			$event_handler = auto_attendant_answer();
		}
		elsif ($event_handler eq  "handle autoattendant") {
			$ctport->off_hook();
			$offhook_status=1;
			$event_handler = auto_attendant();
		}
		else {
			$ctport->logger("main_loop: UNKNOWN HANDLER: $event_handler");
			exit;
		}
	}
}

sub handle_idle() {
	my ($event_source,$event);

	$ctport->logger("handle_idle: Entering Idle State");
	$bparty = -1;
	$ctport->unset_event_handler("incoming_call");
	$ctport->set_event_handler("are u ringing", \&def_are_u_ringing);

	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("handle_idle: event $event");

		if ($event eq "incoming_call") {
			$ctport->logger("handle_idle: Incoming Call from $who");
			$ctport->set_event_handler("incoming_call", \&port_busy);
			$ctport->send_event($who, "call_accepted");
			$bparty = $who;
			return "handle outgoing call";
		}
		elsif ($event eq "ring") {
			$ctport->logger("handle_idle: New incoming external call!");
			$ctport->set_event_handler("incoming_call", \&port_busy);
			$group = $ctport->{CONFIG}{"trunkmap"}{$aparty};
			return "handle incoming call";
		}
	}
}

sub port_busy() {
	my $event_source = shift;
	$ctport->logger("port_busy: Entering Port Busy");
	$ctport->send_event($event_source, "call_not_accepted");    
}

sub def_are_u_ringing() {
	my $event_source = shift;
	$ctport->logger("def_are_u_ringing: Entering");
	$ctport->send_event($event_source, "no");    
}

# DR 8/8/03, changed back to old code, removing LCR for now to simplify
# testing.  Add LCR later when we add it to func spec

sub handle_outgoing_call() {
	my $cont = "continue";
	my $who;

	$ctport->logger("handle_outgoing_call: with $bparty");
	$ctport->off_hook();
	return "handle call";
}

# can handle moving thru groups for incoming calls or blind transfers

sub handle_incoming_call() {
	my $internal = shift;
	my ($who, $event, $extension, $station, $rings, $ext, $tmp);
	my ($port,$max_rings,$voicemail,$pin,$email,$vdir);

	$ctport->logger("handle_incoming_call: Entering Incoming Call State");

	# Check if the $group passed is a [groups]
	my $groups=$ctport->{CONFIG}{groups}{$group};
	if (defined $groups){
		$group=$groups;
	}


	# determine bparty handle
	my $extlist = $group;
	my $found = 0;
	$ctport->logger("handle_incoming_call: Extension list [$extlist]");
	foreach $ext (split /,/,$extlist){

		undef $voicemail;

		# strip off any modifiers
		
		@tmp = split(/:/,$ext);
		$ctport->logger("handle_incoming_call: tmp = @tmp");
		$extension = $tmp[0];
		if ($#tmp) {
		    # modifiers present
		    $ctport->logger("handle_incoming_call: modifier $tmp[1]");
		    @tmp = split(/=/,$tmp[1]); # just assume one modifier for now
		    $ctport->logger("handle_incoming_call: modifier $tmp[0] => $tmp[1]");
		    if ($tmp[0] eq "voicemail") {
			    # can override yes to no or no to yes
			    $voicemail = $tmp[1];
			    $voicemail = "no" unless ($voicemail eq "yes");
		    }
	        }
  
		# check for AA option
		if ($extension =~ /aa/i) {
			if ($offhook_status) {
				$ctport->logger("handle_incoming_call: returning to AA");
				return "handle autoattendant";			
			}
			else {
				$ctport->logger("handle_incoming_call: AA to answer");
				return "handle autoattendant answer";			
			}
		}

		$station = $ctport->{CONFIG}{$extension}{"port"};
		$ctport->logger("handle_incoming_call: trying ext [$extension], port [$station]");
		if (!defined($extension) || !defined($station)) {
			$ctport->logger("handle_incoming_call: group bad - can't continue!");
			return "handle idle";
		}
		    
		# only read per-extension config if not already overridden by modifier
		if (!defined($voicemail)) {
			$voicemail = $ctport->{CONFIG}{$extension}{"voicemail"};
			$voicemail = "no" unless defined($voicemail);
		}

		# see if bparty is busy
		$ret = check_bparty($station);
		if ($ret == 0) {
			$ctport->logger("handle_incoming_call: bparty $extension "
					. "unavailable!");
			$ctport->logger("handle_incoming_call: Checking ext "
					. "$extension for vmail");

			if ($voicemail eq "yes") {
				$ctport->off_hook();
				collect_vmail($extension);
				$ctport->on_hook();
				return "handle idle";
		        }
		}
		else {
			# not busy, lets ring it
			if (defined($internal)) {
				$ret = ring_internal_call($extension, $voicemail);
			}
			else {
				$ret = ring_external_call($extension, $voicemail);
			}
			$ctport->logger("handle_incoming_call: ring ret [$ret]");
			if ($ret ne "ring out") {
				# change state
				return $ret;
			}
			# otherwise lets try next extension
		}
	}

	# we only get to here if all extensions failed
	# returning to idle will cause cycle to repeat itself with next ring
	# which lets us handle "all extensions busy try again" case

	return "handle idle";
}

sub ring_external_call($$) {
	my $extension = shift;
	my $voicemail = shift; # suplied as may be overridden
	my $rings;

	$bparty = $ctport->{CONFIG}{$extension}{"port"};
	my $max_rings = $ctport->{CONFIG}{$extension}{"ringout"};
	$max_rings = 100 unless defined($max_rings);
	
	# OK, other station has accepted call, make it ring

	$ctport->unset_event_handler("are u ringing");
	$rings = 1;
	$ctport->send_event($bparty, "ring");
	$ctport->start_timer_async(10);

	while(1) {
		$ctport->logger("ring_external_call: Waiting for event");
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("ring_external_call: event $event");

		if ($event eq "ring") {

			if ($rings == $max_rings) {
				# we have reached max rings
				$ctport->send_event($bparty, "bye");
				if ($voicemail eq "yes") {
					$ctport->logger("ring_external_call: going to VoiceMail");
					$ctport->off_hook();
					collect_vmail($extension);
					$ctport->on_hook();
					return "handle idle";
				}
				else {
					# tell calling function to try next ext
					return "ring out";
				}

			}
				
			# tell station to ring and reset timer
			$ctport->send_event($bparty, "ring");
			$ctport->start_timer_async(10);
			$rings++;
		}
		elsif ($event eq "timer") {
			# timer has fired, meaning ringing has stopped
			$ctport->send_event($bparty, "bye");
			return "handle idle";
		}
		elsif ($event eq "pickup") {

			if ($who != $bparty) {
				# to handle ** - another ext taking this call
      				# stop current bparty ringing
				# and change to new bparty
				$ctport->send_event($bparty, "bye");
				$bparty = $who;			
			}

			# handle call
			$ctport->off_hook();
			$ctport->bridge($bparty);
			$ctport->set_event_handler("are u ringing", \&def_are_u_ringing);
			return "handle call";
		}
		elsif ($event eq "are u ringing") {
			# another ext is attempting ** to grab call
			$ctport->send_event($who, "yes");
		}
		else {
			$ctport->logger("ring_external_call: Unknown Event $event");
		}
	}
}

sub handle_blind_transfer() {
	my ($extlist, $ext);

	$ctport->logger("handle blind transfer: Entering");

	# look up extension from bparty

	$extlist = $ctport->{CONFIG}{"extmap"}{"extensions"};
	foreach $ext (split /,/,$extlist) {
		$port=$ctport->{CONFIG}{$ext}{"port"};
		if ($port eq $bparty){
		    $extension = $ext;
		}
	}

	if (!defined($extension)) {
		$ctport->on_hook();
		return "handle idle";
	}

	$ctport->logger("handle blind transfer: ext $extension");
	ring_internal_call($extension);
}

# Used for blind transfers, in this case trunk must generate ring-back,
# as it is off-hook already
# Very similar function to handle_incoming_call and ring_internal_call in
# station.pl, but I havent worked out how to re-use code just yet

sub ring_internal_call() {
	my $extension = shift;
	my $voicemail = shift; # suplied as may be overridden
	my $rings;
	my ($who, $event, $tmp, $ext);
	my ($port,$max_rings,$vdir);

	$bparty = $ctport->{CONFIG}{$extension}{"port"};
	$max_rings = $ctport->{CONFIG}{$extension}{"ringout"};
	$max_rings = 100 unless defined($max_rings);

	$ctport->logger("ring_internal_call: Entering Ring Internal Call State");

	# OK, other station has already accepted call, make it ring

	$rings = 1;
	$ctport->send_event($bparty, "ring");
	$ctport->start_timer_async($station_ring_pause);
	$ctport->play_ringback_async();

	while(1) {
		($who,$event) = $ctport->wait_for_event(); 
		$ctport->logger("ring_internal_call: event $event");

		if ($event eq "timer") {
			# timer has fired, ring station again and play ringback tone again

			if ($rings == $max_rings) {
				# we have reached max rings
				$ctport->send_event($bparty, "bye");
				if ($voicemail eq "yes") {
					collect_vmail($extension);
				}

				return "ring out";
			}
				
			$ctport->send_event($bparty, "ring");
			$ctport->start_timer_async($station_ring_pause);
			$ctport->play_ringback_async();
			$rings++;
			$ctport->logger("ring_internal_call: $rings out of $max_rings");
		}
		elsif ($event eq "pickup") {
			# to handle ** - another ext taking this call

			if ($who != $bparty) {
				
				# stop current bparty ringing
				# and change to new bparty
				$ctport->send_event($bparty, "bye");
				$bparty = $who;			
			}

			# handle call
			$ctport->bridge($bparty);
			return "handle call";
		}
		elsif ($event eq "are u ringing") {
			# another ext is attempting ** to grab call
			$ctport->send_event($bparty, "yes");
		}
		elsif ($event eq "loop drop") {
			# caller on trunk port has hung up
			$ctport->send_event($bparty, "bye");
			$ctport->on_hook();
			$offhook_status=0;
			$ctport->stop_timer();
			return "handle idle";
		}
		else {
			$ctport->logger("ring_internal_call: unknownevent $event");
		}
	}
}

sub check_bparty($) {
	my ($bparty)=@_;
	my $who=-1;

	# see if bparty is busy

	$ctport->send_event($bparty, "incoming_call");
	$ctport->start_timer_async(1);

	while(1) {
		($who,$event) = $ctport->wait_for_event(); 

		# all OK
		if ($event eq "call_accepted") {
			$ctport->stop_timer();
			return 1;
		}

		# chosen port busy
		if ($event eq "call_not_accepted") {
			$ctport->stop_timer();
			return 0;
		}

		# no response from bparty after time out
		if ($event eq "timer") {
			return 0;
		}
	}
}

sub handle_call() {
	my ($who,$extra);
	my $ping_count = 0;
	my $ping_time = 30;

	$ctport->logger("handle_call: Entering Handle Call State");

	# fix for spurious loop drop event on off hook transient
	# wait one second, discarding events

	$ctport->start_timer_async(1);
	($who,$event) = $ctport->wait_for_event(); 
	while ($event ne "timer") {
		($who,$event) = $ctport->wait_for_event(); 
		if ($event eq "bye") {
			# bparty has hung up
			$ctport->on_hook();
			$offhook_status=0;
			$ctport->stop_timer();
			return "handle idle";
		}
	}
	
	while(1) {

		($who,$event,$extra) = $ctport->wait_for_event(); 
		$ctport->logger("handle_call: event $event");

		if ($event eq "bye") {
			# station port bparty has hung up
			$ctport->on_hook();
			$offhook_status=0;
			return "handle idle";
		}
		elsif ($event eq "loop drop") {
			$ctport->send_event($bparty, "bye");
			$ctport->on_hook();
			$offhook_status=0;
			return "handle idle";
		}
		elsif ($event eq "bparty"){
			$bparty=$extra;
		}
		elsif ($event eq "hold") {
			# station port bparty tells us to go on hold
			return "handle onhold";
		}
		elsif ($event eq "join") {
			$bparty=$who;
		}
		else {
			$ctport->logger("handle_call: Unknown Event $event");
		}
	}
}

sub handle_onhold(){
	my ($moh, $who, $extra, $event, $mohidx);

	$ctport->logger("handle_onhold: Entering Handle Hold State");

	# safety - after 2 minutes on hold reset trunk
	$ctport->start_timer_async(120);

	$mohidx=0;
	if ($MOH[$mohidx] ne ""){
		$ctport->play_async($MOH[$mohidx]);
		$mohidx++;
	}

	while(1) {
		($who,$event,$extra)= $ctport->wait_for_event(); 
		$ctport->logger("handle_onhold: event $event");

		#other party disappears while we are on hold
		if ($event eq "bye") {
			$ctport->on_hook();
			$offhook_status=0;
			$ctport->play_stop();
			return "handle idle";
		}
		elsif ($event eq "loop drop") {
			$ctport->send_event($bparty, "bye");
			$ctport->on_hook();
			$offhook_status=0;
			$ctport->play_stop();
			return "handle idle";
		}
		elsif ($event eq "unhold") {
			# Tell them we are still about!
			$ctport->play_stop();
			$ctport->send_event($bparty, "unhold_ok");
			return "handle call";
		}
		elsif ($event eq "generate flash") {
			# generate external hook flash
			$ctport->play_stop();
			$ctport->logger("handle_onhold: generating flash");
			$ctport->dial("&");

			# note re-bridge handled on trunk side in this case, so we dont
			# get conflict in use of play hardware resources between
			# dial and bridge functions

			$ctport->bridge($bparty);
			return "handle call";
		}
		elsif ($event eq "playend") {
			$ctport->play_stop();
			if ($mohidx>$#MOH){
				$mohidx=0;
			}	
			if ($MOH[$mohidx] ne ""){
				$ctport->play_async($MOH[$mohidx]);
				$mohidx++;
			}
			else {
				$mohidx=0;
				if ($MOH[$mohidx] ne ""){
					$ctport->play_async($MOH[$mohidx]);
					$mohidx++;
				}
			}
		}
		elsif ($event eq "timer") {
			# hold timeout 
			$ctport->play_stop();
			$ctport->on_hook();
			$offhook_status=0;
			$ctport->send_event($bparty, "bye");
			$ctport->logger("handle_onhold: timer TRUNK RESET");
			return "handle idle";
		}
		elsif ($event eq "unhold_blind") {
			# jump to ring_internal_call state
			$ctport->play_stop();
			return "handle blind transfer";
		}
		elsif ($event eq "bparty") {
			# change bparties, e.g. for blind tfer
			$bparty = $extra;
			$ctport->logger("handle_onhold: new bparty $bparty");
		}
		elsif ($event eq "voicemail") {
			# connect trunk to an extensions voicemail

			$ctport->play_stop();
			$extension = $extra;
			$ctport->logger("handle_onhold: sent to voicemail ext: $extension");

			if (defined($extension)) {
				# note we override "voicemail=no" switch
				collect_vmail($extension);
			}

			return "handle idle";
		}
		else {
			$ctport->logger("handle_onhold: Unknown Event $event");
		}
	}
}


